<?php

namespace Drupal\imds4azure_mysql\Driver\Database\imds4azure_mysql;

use Drupal\mysql\Driver\Database\mysql\Connection as CoreConnection;
use Flossiraptor\Imds4azure\IMDS;

/**
 * Extension of the MySQL connection which injects the Azure MI token.
 */
class Connection extends CoreConnection {

  /**
   * Default provider for the IMDS service. This can be overridden in settings.
   */
  const DEFAULT_IMDS_PROVIDER = IMDS::class;

  /**
   * The default Azure resource is a MySQL Flexible Server.
   */
  const DEFAULT_RESOURCE = 'https://ossrdbms-aad.database.windows.net/';

  /**
   * {@inheritdoc}
   */
  public static function open(array &$connection_options = []) {
    $connection_options['password'] = self::getToken($connection_options);
    return parent::open($connection_options);
  }

  /**
   * Get the authentication token for the requested scope.
   */
  public static function getToken(array &$connection_options = []) : string {
    static $token = NULL;

    if (is_null($token)) {
      /** @var \Flossiraptor\Imds4azure\IMDS */
      $imds = self::getImds($connection_options);

      // Request the managed-identity token from the IMDS4Azure service.
      // This will be passed as the password for the DB connection.
      $token = $imds->identity()->getToken(
        resource: $connection_options['imds4azure']['resource'] ?? self::DEFAULT_RESOURCE,
        client_id: $connection_options['imds4azure']['client_id'] ?? NULL,
      );
    }
    return (string) $token;
  }

  /**
   * Get the IMDS library.
   *
   * @param array $connection_options
   *   The database connection settings array.
   *
   * @return \Flossiraptor\Imds4azure\IMDS
   *   An IMDS service.
   */
  public static function getImds(array &$connection_options = []) : IMDS {
    $provider = self::DEFAULT_IMDS_PROVIDER;
    if (array_key_exists('imds4azure', $connection_options)) {
      if (array_key_exists('imds_provider', $connection_options['imds4azure'])) {
        $provider = $connection_options['imds4azure']['provider'];
      }
    }
    return new $provider();
  }

}
