<?php

namespace Drupal\imds4azure\Drush\Commands;

use Consolidation\OutputFormatters\StructuredData\PropertyList;
use Drush\Attributes as CLI;
use Drush\Boot\DrupalBootLevels;
use Drush\Commands\DrushCommands;
use Flossiraptor\Imds4azure\IMDS;
use Psr\Container\ContainerInterface as DrushContainer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Drush integration to query IMDS and fetch tokens.
 */
class Imds4AzureDrushCommands extends DrushCommands {

  /**
   * Constructor.
   *
   * @param \Flossiraptor\Imds4azure\IMDS $imds
   *   The IMDS client.
   */
  public function __construct(protected IMDS $imds) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, DrushContainer $drush): self {
    return new static(
      $container->get('imds4azure.imds'),
    );
  }

  /**
   * Get an authentication token from the IMDS identity service.
   *
   * @param string $resource
   *   The URL of the Azure resource.
   * @param string $client_id
   *   (optional) The ID of the Managed Identity to use.
   * @param array $options
   *   (optional) The options specified by Drush.
   *
   * @return \Consolidation\OutputFormatters\StructuredData\PropertyList
   *   The authentication token, structured for output by Drush.
   */
  #[CLI\Bootstrap(DrupalBootLevels::CONFIGURATION)]
  #[CLI\Command(name: 'imds:get-token')]
  #[CLI\Argument(name: 'resource', description: 'URL for the azure resource')]
  #[CLI\Argument(name: 'client_id', description: 'Optional Managed Identity client ID to provide. This should be a UUID.')]
  #[CLI\DefaultFields(fields: ['access_token'])]
  #[CLI\FieldLabels(labels: [
    'access_token' => 'Token',
    'expires_in' => 'Valid for',
    'expires_on' => 'Expiry',
    'client_id' => 'Client ID',
    'resource' => 'Resource',
    'token_type' => 'Token type',
  ])]
  public function getToken(
    string $resource,
    ?string $client_id = NULL,
    array $options = ['format' => 'string'],
  ) : PropertyList {
    $token = $this
      ->imds
      ->identity()
      ->getToken($resource, $client_id)
      ->getToken();

    return new PropertyList($token);
  }

}
