<?php

namespace Drupal\imds4azure\Service;

use Flossiraptor\Imds4azure\IMDS;
use Flossiraptor\Imds4azure\Service\Identity;
use Flossiraptor\Imds4azure\Service\Instance;
use Flossiraptor\Imds4azure\Service\LoadBalancer;

/**
 * Factory to create IMDS clients for Drupal.
 */
interface ImdsClientFactoryInterface {

  /**
   * The IMDS service manages all IMDS requests and APIs.
   *
   * @return \Flossiraptor\Imds4azure\IMDS
   *   The IMDS service.
   */
  public static function imds() : IMDS;

  /**
   * The Identity service can request Managed Identity tokens.
   *
   * @return \Flossiraptor\Imds4azure\Service\Identity
   *   The service which requests Managed Identity tokens.
   */
  public static function identity() : Identity;

  /**
   * The Instance service can look up metadata about the VM/container instance.
   *
   * @return \Flossiraptor\Imds4azure\Service\Instance
   *   The service which reports workload metadata.
   */
  public static function instance() : Instance;

  /**
   * The LoadBalancer service can provide information about connected gateways.
   *
   * @return \Flossiraptor\Imds4azure\Service\LoadBalancer
   *   The service which provides LoadBalancer information.
   */
  public static function loadbalancer() : LoadBalancer;

  /**
   * Alias for the `instance` service.
   *
   * @return \Flossiraptor\Imds4azure\Service\Instance
   *   The service which reports workload metadata.
   */
  public static function metadata() : Instance;

}
